@@ -73,11 +73,28 @@ class TestInteractiveMode(base.TestBase):
         app.interactive_app_factory = mock.MagicMock(
             name='interactive_app_factory'
         )
-        self.assertIs(None, app.interpreter)
-        app.run([])
-        self.assertIsNot(None, app.interpreter)
+        self.assertIsNone(app.interpreter)
+        ret = app.run([])
+        self.assertIsNotNone(app.interpreter)
         cmdloop = app.interactive_app_factory.return_value.cmdloop
         cmdloop.assert_called_once_with()
+        self.assertNotEqual(ret, 0)
+
+    def test_interactive_mode_cmdloop_error(self):
+        app, command = make_app()
+        cmdloop_mock = mock.MagicMock(
+            name='cmdloop',
+        )
+        cmdloop_mock.return_value = 1
+        app.interactive_app_factory = mock.MagicMock(
+            name='interactive_app_factory'
+        )
+        self.assertIsNone(app.interpreter)
+        ret = app.run([])
+        self.assertIsNotNone(app.interpreter)
+        cmdloop = app.interactive_app_factory.return_value.cmdloop
+        cmdloop.assert_called_once_with()
+        self.assertNotEqual(ret, 0)
 
 
 class TestInitAndCleanup(base.TestBase):
@@ -99,14 +116,16 @@ class TestInitAndCleanup(base.TestBase):
     def test_clean_up_success(self):
         app, command = make_app()
         app.clean_up = mock.MagicMock(name='clean_up')
-        app.run(['mock'])
+        ret = app.run(['mock'])
         app.clean_up.assert_called_once_with(command.return_value, 0, None)
+        self.assertEqual(ret, 0)
 
     def test_clean_up_error(self):
         app, command = make_app()
 
         app.clean_up = mock.MagicMock(name='clean_up')
-        app.run(['error'])
+        ret = app.run(['error'])
+        self.assertNotEqual(ret, 0)
 
         app.clean_up.assert_called_once_with(mock.ANY, mock.ANY, mock.ANY)
         call_args = app.clean_up.call_args_list[0]
@@ -119,12 +138,8 @@ class TestInitAndCleanup(base.TestBase):
         app, command = make_app()
 
         app.clean_up = mock.MagicMock(name='clean_up')
-        try:
-            app.run(['--debug', 'error'])
-        except RuntimeError as err:
-            self.assertIs(err, app.clean_up.call_args_list[0][0][2])
-        else:
-            self.fail('Should have had an exception')
+        ret = app.run(['--debug', 'error'])
+        self.assertNotEqual(ret, 0)
 
         self.assertTrue(app.clean_up.called)
         call_args = app.clean_up.call_args_list[0]
@@ -157,7 +172,7 @@ class TestInitAndCleanup(base.TestBase):
             side_effect=RuntimeError('within clean_up'),
         )
         try:
-            app.run(['--debug', 'error'])
+            ret = app.run(['--debug', 'error'])
         except RuntimeError as err:
             if not hasattr(err, '__context__'):
                 # The exception passed to clean_up is not the exception
@@ -166,7 +181,7 @@ class TestInitAndCleanup(base.TestBase):
                 # with the new one as a __context__ attribute.
                 self.assertIsNot(err, app.clean_up.call_args_list[0][0][2])
         else:
-            self.fail('Should have had an exception')
+            self.assertNotEqual(ret, 0)
 
         self.assertTrue(app.clean_up.called)
         call_args = app.clean_up.call_args_list[0]
@@ -498,3 +513,53 @@ class TestIO(base.TestBase):
             self.assertIs(sys.stdin, app.stdin)
             self.assertIs(sys.stdout, app.stdout)
             self.assertIs(io, app.stderr)
+
+    def test_writer_encoding(self):
+        # The word "test" with the e replaced by
+        # Unicode latin small letter e with acute,
+        # U+00E9, utf-8 encoded as 0xC3 0xA9
+        text = u't\u00E9st'
+        text_utf8 = text.encode('utf-8')
+
+        if six.PY2:
+            # In PY2 StreamWriter can't accept non-ASCII encoded characters
+            # because it must first promote the encoded byte stream to
+            # unicode in order to encode it in the desired encoding.
+            # Because the encoding of the byte stream is not known at this
+            # point the default-encoding of ASCII is utilized, but you can't
+            # decode a non-ASCII charcater to ASCII.
+            io = six.StringIO()
+            writer = codecs.getwriter('utf-8')(io)
+            self.assertRaises(UnicodeDecodeError,
+                              writer.write,
+                              text_utf8)
+
+            # In PY2 with our override of codecs.getwriter we do not
+            # attempt to encode bytes in a str object (only unicode
+            # objects) therefore the final output string should be the
+            # utf-8 encoded byte sequence
+            io = six.StringIO()
+            writer = utils.getwriter('utf-8')(io)
+            writer.write(text)
+            output = io.getvalue()
+            self.assertEqual(text_utf8, output)
+
+            io = six.StringIO()
+            writer = utils.getwriter('utf-8')(io)
+            writer.write(text_utf8)
+            output = io.getvalue()
+            self.assertEqual(text_utf8, output)
+        else:
+            # In PY3 you can't write encoded bytes to a text writer
+            # instead text functions require text.
+            io = six.StringIO()
+            writer = utils.getwriter('utf-8')(io)
+            self.assertRaises(TypeError,
+                              writer.write,
+                              text)
+
+            io = six.StringIO()
+            writer = utils.getwriter('utf-8')(io)
+            self.assertRaises(TypeError,
+                              writer.write,
+                              text_utf8)
